import { Logger, OnModuleDestroy } from '@nestjs/common';
import { I18nOptions, I18nValidationError } from '..';
import { I18nTranslation } from '../interfaces/i18n-translation.interface';
import { Observable, BehaviorSubject } from 'rxjs';
import { I18nLoader } from '../loaders/i18n.loader';
export declare type TranslateOptions = {
    lang?: string;
    args?: ({
        [k: string]: any;
    } | string)[] | {
        [k: string]: any;
    };
    defaultValue?: string;
    debug?: boolean;
};
export declare class I18nService implements OnModuleDestroy {
    private readonly i18nOptions;
    private readonly logger;
    private readonly loader;
    private readonly languagesSubject;
    private readonly translationsSubject;
    private supportedLanguages;
    private translations;
    private pluralRules;
    private unsubscribe;
    constructor(i18nOptions: I18nOptions, translations: Observable<I18nTranslation>, supportedLanguages: Observable<string[]>, logger: Logger, loader: I18nLoader, languagesSubject: BehaviorSubject<string[]>, translationsSubject: BehaviorSubject<I18nTranslation>);
    onModuleDestroy(): void;
    translate<T = any>(key: string, options?: TranslateOptions): T;
    t<T = any>(key: string, options?: TranslateOptions): T;
    getSupportedLanguages(): string[];
    getTranslations(): I18nTranslation;
    refresh(translations?: I18nTranslation | Observable<I18nTranslation>, languages?: string[] | Observable<string[]>): Promise<void>;
    hbsHelper: (key: string, args: any, options: any) => any;
    private translateObject;
    resolveLanguage(lang: string): string;
    private getPluralObject;
    private getNestedTranslations;
    validate(value: any, options?: TranslateOptions): Promise<I18nValidationError[]>;
}
